@@ -13,17 +13,18 @@ from typing import Sequence
 from typing import Tuple
 from typing import Union
 
-import py
+import py.path
 
 import pytest
 from _pytest import outcomes
 from _pytest._code.code import ExceptionInfo
 from _pytest._code.code import ReprFileLocation
 from _pytest._code.code import TerminalRepr
+from _pytest._io import TerminalWriter
 from _pytest.compat import safe_getattr
 from _pytest.compat import TYPE_CHECKING
 from _pytest.fixtures import FixtureRequest
-from _pytest.outcomes import Skipped
+from _pytest.outcomes import OutcomeException
 from _pytest.python_api import approx
 from _pytest.warning_types import PytestWarning
 
@@ -111,9 +112,9 @@ def pytest_collect_file(path, parent):
     config = parent.config
     if path.ext == ".py":
         if config.option.doctestmodules and not _is_setup_py(config, path, parent):
-            return DoctestModule(path, parent)
+            return DoctestModule.from_parent(parent, fspath=path)
     elif _is_doctest(config, path, parent):
-        return DoctestTextfile(path, parent)
+        return DoctestTextfile.from_parent(parent, fspath=path)
 
 
 def _is_setup_py(config, path, parent):
@@ -139,7 +140,7 @@ class ReprFailDoctest(TerminalRepr):
     ):
         self.reprlocation_lines = reprlocation_lines
 
-    def toterminal(self, tw: py.io.TerminalWriter) -> None:
+    def toterminal(self, tw: TerminalWriter) -> None:
         for reprlocation, lines in self.reprlocation_lines:
             for line in lines:
                 tw.line(line)
@@ -177,7 +178,7 @@ def _init_runner_class() -> "Type[doctest.DocTestRunner]":
                 raise failure
 
         def report_unexpected_exception(self, out, test, example, exc_info):
-            if isinstance(exc_info[1], Skipped):
+            if isinstance(exc_info[1], OutcomeException):
                 raise exc_info[1]
             if isinstance(exc_info[1], bdb.BdbQuit):
                 outcomes.exit("Quitting debugger")
@@ -218,6 +219,16 @@ class DoctestItem(pytest.Item):
         self.obj = None
         self.fixture_request = None
 
+    @classmethod
+    def from_parent(  # type: ignore
+        cls, parent: "Union[DoctestTextfile, DoctestModule]", *, name, runner, dtest
+    ):
+        # incompatible signature due to to imposed limits on sublcass
+        """
+        the public named constructor
+        """
+        return super().from_parent(name=name, parent=parent, runner=runner, dtest=dtest)
+
     def setup(self):
         if self.dtest is not None:
             self.fixture_request = _setup_fixtures(self)
@@ -228,7 +239,7 @@ class DoctestItem(pytest.Item):
                 globs[name] = value
             self.dtest.globs.update(globs)
 
-    def runtest(self):
+    def runtest(self) -> None:
         _check_all_skipped(self.dtest)
         self._disable_output_capturing_for_darwin()
         failures = []  # type: List[doctest.DocTestFailure]
@@ -302,13 +313,16 @@ class DoctestItem(pytest.Item):
                 else:
                     inner_excinfo = ExceptionInfo(failure.exc_info)
                     lines += ["UNEXPECTED EXCEPTION: %s" % repr(inner_excinfo.value)]
-                    lines += traceback.format_exception(*failure.exc_info)
+                    lines += [
+                        x.strip("\n")
+                        for x in traceback.format_exception(*failure.exc_info)
+                    ]
                 reprlocation_lines.append((reprlocation, lines))
             return ReprFailDoctest(reprlocation_lines)
         else:
             return super().repr_failure(excinfo)
 
-    def reportinfo(self) -> Tuple[str, int, str]:
+    def reportinfo(self) -> Tuple[py.path.local, int, str]:
         return self.fspath, self.dtest.lineno, "[doctest] %s" % self.name
 
 
@@ -373,7 +387,9 @@ class DoctestTextfile(pytest.Module):
         parser = doctest.DocTestParser()
         test = parser.get_doctest(text, globs, name, filename, 0)
         if test.examples:
-            yield DoctestItem(test.name, self, runner, test)
+            yield DoctestItem.from_parent(
+                self, name=test.name, runner=runner, dtest=test
+            )
 
 
 def _check_all_skipped(test):
@@ -482,7 +498,9 @@ class DoctestModule(pytest.Module):
 
         for test in finder.find(module, module.__name__):
             if test.examples:  # skip empty doctests
-                yield DoctestItem(test.name, self, runner, test)
+                yield DoctestItem.from_parent(
+                    self, name=test.name, runner=runner, dtest=test
+                )
 
 
 def _setup_fixtures(doctest_item):
