@@ -5,12 +5,16 @@ from contextlib import contextmanager
 from io import StringIO
 from typing import AbstractSet
 from typing import Dict
+from typing import Generator
 from typing import List
 from typing import Mapping
+from typing import Optional
 
 import pytest
+from _pytest import nodes
 from _pytest.compat import nullcontext
 from _pytest.config import _strtobool
+from _pytest.config import Config
 from _pytest.config import create_terminal_writer
 from _pytest.pathlib import Path
 
@@ -192,7 +196,12 @@ def pytest_addoption(parser):
         "--log-level",
         dest="log_level",
         default=None,
-        help="logging level used by the logging module",
+        metavar="LEVEL",
+        help=(
+            "level of messages to catch/display.\n"
+            "Not set by default, so it depends on the root/parent log handler's"
+            ' effective level, where it is "WARNING" by default.'
+        ),
     )
     add_option_ini(
         "--log-format",
@@ -325,13 +334,13 @@ class LogCaptureFixture:
             logger.setLevel(level)
 
     @property
-    def handler(self):
+    def handler(self) -> LogCaptureHandler:
         """
         :rtype: LogCaptureHandler
         """
-        return self._item.catch_log_handler
+        return self._item.catch_log_handler  # type: ignore[no-any-return]  # noqa: F723
 
-    def get_records(self, when):
+    def get_records(self, when: str) -> List[logging.LogRecord]:
         """
         Get the logging records for one of the possible test phases.
 
@@ -345,7 +354,7 @@ class LogCaptureFixture:
         """
         handler = self._item.catch_log_handlers.get(when)
         if handler:
-            return handler.records
+            return handler.records  # type: ignore[no-any-return]  # noqa: F723
         else:
             return []
 
@@ -441,9 +450,7 @@ def caplog(request):
     result._finalize()
 
 
-def get_actual_log_level(config, *setting_names):
-    """Return the actual logging level."""
-
+def get_log_level_for_setting(config: Config, *setting_names: str) -> Optional[int]:
     for setting_name in setting_names:
         log_level = config.getoption(setting_name)
         if log_level is None:
@@ -451,7 +458,7 @@ def get_actual_log_level(config, *setting_names):
         if log_level:
             break
     else:
-        return
+        return None
 
     if isinstance(log_level, str):
         log_level = log_level.upper()
@@ -476,7 +483,7 @@ class LoggingPlugin:
     """Attaches to the logging module and captures log messages for each test.
     """
 
-    def __init__(self, config):
+    def __init__(self, config: Config) -> None:
         """Creates a new plugin to capture log messages.
 
         The formatter can be safely shared across all handlers so
@@ -485,14 +492,20 @@ class LoggingPlugin:
         self._config = config
 
         self.print_logs = get_option_ini(config, "log_print")
+        if not self.print_logs:
+            from _pytest.warnings import _issue_warning_captured
+            from _pytest.deprecated import NO_PRINT_LOGS
+
+            _issue_warning_captured(NO_PRINT_LOGS, self._config.hook, stacklevel=2)
+
         self.formatter = self._create_formatter(
             get_option_ini(config, "log_format"),
             get_option_ini(config, "log_date_format"),
             get_option_ini(config, "log_auto_indent"),
         )
-        self.log_level = get_actual_log_level(config, "log_level")
+        self.log_level = get_log_level_for_setting(config, "log_level")
 
-        self.log_file_level = get_actual_log_level(config, "log_file_level")
+        self.log_file_level = get_log_level_for_setting(config, "log_file_level")
         self.log_file_format = get_option_ini(config, "log_file_format", "log_format")
         self.log_file_date_format = get_option_ini(
             config, "log_file_date_format", "log_date_format"
@@ -505,7 +518,7 @@ class LoggingPlugin:
         if log_file:
             self.log_file_handler = logging.FileHandler(
                 log_file, mode="w", encoding="UTF-8"
-            )
+            )  # type: Optional[logging.FileHandler]
             self.log_file_handler.setFormatter(self.log_file_formatter)
         else:
             self.log_file_handler = None
@@ -555,7 +568,7 @@ class LoggingPlugin:
             get_option_ini(config, "log_auto_indent"),
         )
 
-        log_cli_level = get_actual_log_level(config, "log_cli_level", "log_level")
+        log_cli_level = get_log_level_for_setting(config, "log_cli_level", "log_level")
         self.log_cli_handler = log_cli_handler
         self.live_logs_context = lambda: catching_logs(
             log_cli_handler, formatter=log_cli_formatter, level=log_cli_level
@@ -591,7 +604,7 @@ class LoggingPlugin:
         ) is not None or self._config.getini("log_cli")
 
     @pytest.hookimpl(hookwrapper=True, tryfirst=True)
-    def pytest_collection(self):
+    def pytest_collection(self) -> Generator[None, None, None]:
         with self.live_logs_context():
             if self.log_cli_handler:
                 self.log_cli_handler.set_when("collection")
@@ -612,7 +625,9 @@ class LoggingPlugin:
                 yield
 
     @contextmanager
-    def _runtest_for_main(self, item, when):
+    def _runtest_for_main(
+        self, item: nodes.Item, when: str
+    ) -> Generator[None, None, None]:
         """Implements the internals of pytest_runtest_xxx() hook."""
         with catching_logs(
             LogCaptureHandler(), formatter=self.formatter, level=self.log_level
@@ -625,15 +640,15 @@ class LoggingPlugin:
                 return
 
             if not hasattr(item, "catch_log_handlers"):
-                item.catch_log_handlers = {}
-            item.catch_log_handlers[when] = log_handler
-            item.catch_log_handler = log_handler
+                item.catch_log_handlers = {}  # type: ignore[attr-defined]  # noqa: F821
+            item.catch_log_handlers[when] = log_handler  # type: ignore[attr-defined]  # noqa: F821
+            item.catch_log_handler = log_handler  # type: ignore[attr-defined]  # noqa: F821
             try:
                 yield  # run test
             finally:
                 if when == "teardown":
-                    del item.catch_log_handler
-                    del item.catch_log_handlers
+                    del item.catch_log_handler  # type: ignore[attr-defined]  # noqa: F821
+                    del item.catch_log_handlers  # type: ignore[attr-defined]  # noqa: F821
 
             if self.print_logs:
                 # Add a captured log section to the report.
