@@ -1,4 +1,5 @@
 """ discovery and running of std-library "unittest" style tests. """
+import functools
 import sys
 import traceback
 
@@ -13,6 +14,8 @@ from _pytest.outcomes import xfail
 from _pytest.python import Class
 from _pytest.python import Function
 from _pytest.runner import CallInfo
+from _pytest.skipping import skipped_by_mark_key
+from _pytest.skipping import unexpectedsuccess_key
 
 
 def pytest_pycollect_makeitem(collector, name, obj):
@@ -23,7 +26,7 @@ def pytest_pycollect_makeitem(collector, name, obj):
     except Exception:
         return
     # yes, so let's collect it
-    return UnitTestCase(name, parent=collector)
+    return UnitTestCase.from_parent(collector, name=name, obj=obj)
 
 
 class UnitTestCase(Class):
@@ -51,7 +54,7 @@ class UnitTestCase(Class):
             if not getattr(x, "__test__", True):
                 continue
             funcobj = getimfunc(x)
-            yield TestCaseFunction(name, parent=self, callobj=funcobj)
+            yield TestCaseFunction.from_parent(self, name=name, callobj=funcobj)
             foundsomething = True
 
         if not foundsomething:
@@ -59,7 +62,8 @@ class UnitTestCase(Class):
             if runtest is not None:
                 ut = sys.modules.get("twisted.trial.unittest", None)
                 if ut is None or runtest != ut.TestCase.runTest:
-                    yield TestCaseFunction("runTest", parent=self)
+                    # TODO: callobj consistency
+                    yield TestCaseFunction.from_parent(self, name="runTest")
 
     def _inject_setup_teardown_fixtures(self, cls):
         """Injects a hidden auto-use fixture to invoke setUpClass/setup_method and corresponding
@@ -109,12 +113,15 @@ class TestCaseFunction(Function):
     _testcase = None
 
     def setup(self):
+        self._needs_explicit_tearDown = False
         self._testcase = self.parent.obj(self.name)
         self._obj = getattr(self._testcase, self.name)
         if hasattr(self, "_request"):
             self._request._fillfixtures()
 
     def teardown(self):
+        if self._needs_explicit_tearDown:
+            self._testcase.tearDown()
         self._testcase = None
         self._obj = None
 
@@ -169,7 +176,7 @@ class TestCaseFunction(Function):
         try:
             skip(reason)
         except skip.Exception:
-            self._skipped_by_mark = True
+            self._store[skipped_by_mark_key] = True
             self._addexcinfo(sys.exc_info())
 
     def addExpectedFailure(self, testcase, rawexcinfo, reason=""):
@@ -179,7 +186,7 @@ class TestCaseFunction(Function):
             self._addexcinfo(sys.exc_info())
 
     def addUnexpectedSuccess(self, testcase, reason=""):
-        self._unexpectedsuccess = reason
+        self._store[unexpectedsuccess_key] = reason
 
     def addSuccess(self, testcase):
         pass
@@ -187,29 +194,46 @@ class TestCaseFunction(Function):
     def stopTest(self, testcase):
         pass
 
-    def _handle_skip(self):
-        # implements the skipping machinery (see #2137)
-        # analog to pythons Lib/unittest/case.py:run
-        testMethod = getattr(self._testcase, self._testcase._testMethodName)
-        if getattr(self._testcase.__class__, "__unittest_skip__", False) or getattr(
-            testMethod, "__unittest_skip__", False
-        ):
-            # If the class or method was skipped.
-            skip_why = getattr(
-                self._testcase.__class__, "__unittest_skip_why__", ""
-            ) or getattr(testMethod, "__unittest_skip_why__", "")
-            self._testcase._addSkip(self, self._testcase, skip_why)
-            return True
-        return False
+    def _expecting_failure(self, test_method) -> bool:
+        """Return True if the given unittest method (or the entire class) is marked
+        with @expectedFailure"""
+        expecting_failure_method = getattr(
+            test_method, "__unittest_expecting_failure__", False
+        )
+        expecting_failure_class = getattr(self, "__unittest_expecting_failure__", False)
+        return bool(expecting_failure_class or expecting_failure_method)
 
     def runtest(self):
-        if self.config.pluginmanager.get_plugin("pdbinvoke") is None:
+        # TODO: move testcase reporter into separate class, this shouldnt be on item
+        import unittest
+
+        testMethod = getattr(self._testcase, self._testcase._testMethodName)
+
+        class _GetOutOf_testPartExecutor(KeyboardInterrupt):
+            """Helper exception to get out of unittests's testPartExecutor (see TestCase.run)."""
+
+        @functools.wraps(testMethod)
+        def wrapped_testMethod(*args, **kwargs):
+            """Wrap the original method to call into pytest's machinery, so other pytest
+            features can have a chance to kick in (notably --pdb)"""
+            try:
+                self.ihook.pytest_pyfunc_call(pyfuncitem=self)
+            except unittest.SkipTest:
+                raise
+            except Exception as exc:
+                expecting_failure = self._expecting_failure(testMethod)
+                if expecting_failure:
+                    raise
+                self._needs_explicit_tearDown = True
+                raise _GetOutOf_testPartExecutor(exc)
+
+        setattr(self._testcase, self._testcase._testMethodName, wrapped_testMethod)
+        try:
             self._testcase(result=self)
-        else:
-            # disables tearDown and cleanups for post mortem debugging (see #1890)
-            if self._handle_skip():
-                return
-            self._testcase.debug()
+        except _GetOutOf_testPartExecutor as exc:
+            raise exc.args[0] from exc.args[0]
+        finally:
+            delattr(self._testcase, self._testcase._testMethodName)
 
     def _prunetraceback(self, excinfo):
         Function._prunetraceback(self, excinfo)
